use crate::read;

pub fn run() {
    let grid: Vec<Vec<u32>> = read("08")
        .lines()
        .map(|l| {
            l.chars()
                .map(|c| c.to_digit(10).unwrap())
                .collect::<Vec<_>>()
        })
        .collect::<Vec<_>>();

    let width = grid.first().unwrap().len();

    let res = grid
        .iter()
        .enumerate()
        .skip(1)
        .flat_map(|(row, row_vec)| {
            row_vec
                .iter()
                .enumerate()
                .skip(1)
                .map(|(col, height)| {
                    let mut sides_visible = 4;
                    let mut score = 1;

                    score *= if let Some(sc_score) = (0..col)
                        .rev()
                        .map(|cc| (col - cc, row_vec[cc]))
                        .find(|(_sc_score, other_height)| other_height >= height)
                    {
                        sides_visible -= 1;
                        sc_score.0
                    } else {
                        col
                    };

                    score *= if let Some(sc_score) = (col + 1..width)
                        .map(|cc| (cc - col, row_vec[cc]))
                        .find(|(_sc_score, other_height)| other_height >= height)
                    {
                        sides_visible -= 1;
                        sc_score.0
                    } else {
                        width - col - 1
                    };

                    score *= if let Some(sc_score) = (0..row)
                        .rev()
                        .map(|rr| (row - rr, grid[rr][col]))
                        .find(|(_sc_score, other_height)| other_height >= height)
                    {
                        sides_visible -= 1;
                        sc_score.0
                    } else {
                        row
                    };

                    score *= if let Some(sc_score) = grid
                        .iter()
                        .enumerate()
                        .skip(row + 1)
                        .map(|(rr, l)| (rr - row, l[col]))
                        .find(|(_sc_score, other_height)| other_height >= height)
                    {
                        sides_visible -= 1;
                        sc_score.0
                    } else {
                        grid.len() - row - 1
                    };

                    (sides_visible > 0, score)
                })
                .collect::<Vec<_>>()
        })
        .collect::<Vec<_>>();

    #[cfg(feature = "part1")]
    {
        let count = res.iter().filter(|e| e.0).count();
        println!("Day 8, Part 01: {}", count);
    }

    #[cfg(feature = "part2")]
    {
        let max_score = res.iter().map(|e| e.1).max().unwrap();
        println!("Day 8, Part 02: {}", max_score);
    }
}
